/*
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 2 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */
#include <string.h>
#include <stdlib.h>
#include <gtk/gtk.h>
#include "load.h"
#include "free.h"
#include "alloc.h"
#include "draw.h"
#include "throw.h"
#include "game.h"
#include "position.h"
#include "select.h"
#include "set.h"
#include "get.h"
#include "def.h"

/* click_givecard : takes care of distributing 
 * the first batch of cards to the players 
 * (5 cards each) and show one more card. This 
 * final card will be proposed to each player 
 * one at a time. If anyone decide to take it, 
 * then the family of this card will become the 
 * master family of the round (trump).
 * 
 * struct _prog *prog_data: see file def.h 
 * 
 * What is needed in struct _prog : 
 * 
 * prog_data->state : the status of the game 
 * right now. Are we playing yet ? 
 * This is an enum egstate (see file def.h 
 * for more details about it)
 * 
 * prog_data->allwidgets[4] : allwidgets is 
 * an array composed by many GtkWidget* each.
 * The Fifth ([4]) is a spinbutton; which will 
 * be suppressed in the future.
 */
void click_givecard(struct _prog *prog_data)
{
	/*gtk_widget_set_sensitive(prog_data->button, FALSE);*/
	gtk_label_set_text(GTK_LABEL(prog_data->allwidgets[6]), NULL);
	gtk_button_set_label(GTK_BUTTON(prog_data->allwidgets[2]), "Pass");

	prog_data->state = DISTRIBUTING;

	game_distribute_phase1(prog_data);
	
	game_finish_phase1(prog_data);

	prog_data->player = prog_data->distributer;
	
	select_player_next(&prog_data->player);
	
	prog_data->menext = prog_data->player;
	
	prog_data->state = READY_DISTRIB1;
	
	if(prog_data->output != NULL)
	{
		fprintf(prog_data->output,
			"Ready");
	}

	game_choose_trump(prog_data);
	/*
	gtk_widget_set_sensitive(prog_data->allwidgets[4], TRUE);
	gtk_widget_set_sensitive(prog_data->button, TRUE);*/
}

/* click_selectmaster : Start the first turn of this round.
 * This means many things has to be done. First of all 
 * when calling this function it means a player decided 
 * which master family is the trump of this round. 
 * We need to set this, to send the remaining cards 
 * to each player (3 each, except the so called master 
 * of the game, he will get 2 from the pile and one 
 * which was the card proposed to the players as the 
 * trump). Then we needs to set new points for all 
 * cards from the trump family. They are now more 
 * powerful then they were initialised.
 * Then the computer will play he has to.
 * 
 * struct _prog *prog_data: see file def.h
 * 
 * What is needed in struct _prog:
 * 
 * prog_data->state : an enum egstate, see 
 * file def.h for more details
 * 
 * prog_data->master : who takes the proposed 
 * card. 
 * 
 * prog_data->player : who has to drop a card 
 * right now. It's not exactly always this. 
 * When each players has dropped one card, 
 * the player is not the player who needs to 
 * drop a card next. --Need to be Fixed--
 * 
 * prog_data->hashand : who is in charge 
 * of deciding the first of each four cards 
 * to be dropped on the table at each turn.
 * This is the player who has the hand 
 * before anyone drops a more powerful card.
 * Note: it's not chaning when someone do 
 * this (dropping a more powerful card). 
 * That is why it's called "hashand" and not 
 * "havehand". See menext for the player who 
 * have the hand right now. 
 * 
 * prog_data->menext : the player which 
 * win this turn at the moment. It can 
 * be changed at any card drop except the 
 * first. When you win a turn, you are 
 * the first to drop the first card next.
 * This is exactly when menext should 
 * not be changed. You can't change menext 
 * at this time because you are the one 
 * to drop the card. You are then already 
 * the menext player.
 * 
 * prog_data->distributer : which player 
 * takes care of distributing the cards. 
 * Note: you don't have to do it with 
 * the mouse or something like this. This 
 * is all virtual dude. Anyway it's needed 
 * to know who is the first to play on the 
 * first turn. 
 * Note: each round it's incremented by 
 * turning round the clock.
 * 
 * prog_data->family : the trump color. 
 * This is not the proposed card every time. 
 * --Need to be Fixed--
 * 
 * prog_data->allwidgets[4] : allwidgets is an 
 * array composed by many GtkWidget*.
 * The fifth is the spinbutton, which will 
 * be suppressed in the future.
 * 
 * prog_data->waiting : a GList representing 
 * the table. This is where you drop your cards.
 * Each element of this list is a struct _card, 
 * defined in the def.h file.
 * 
 * prog_data->allwidgets[5] : a GtkImage 
 * containing nothing at first. This is where 
 * we put the icon of the trump family 
 * (diamond, heart, spade or club)
 * 
 * prog_data->icons : an array of 4 GtkImage, 
 * each of the family icons.
 * 
 * prog_data->output : the stream to print 
 * messages to. Default is stdout.
 */
void click_selectmaster(struct _prog *prog_data, int master, int trump)
{
	char str[10];
	
	prog_data->state = DISTRIBUTING;
	
	prog_data->master = master;
	
	switch(master)
	{
		case 0:
		strcpy(str, "Top");
		break;
		case 1:
		strcpy(str, "Right");
		break;
		case 2:
		strcpy(str, "You");
		break;
		case 3:
		strcpy(str, "Left");
		break;
	}
	
	gtk_label_set_text(GTK_LABEL(prog_data->allwidgets[7]), str);
	
	prog_data->family = trump;
	
	prog_data->player = prog_data->menext;
	prog_data->hashand = prog_data->menext;
	
	gtk_image_set_from_pixbuf(GTK_IMAGE(prog_data->allwidgets[5]),
		prog_data->icons[prog_data->family]);
	
	if(prog_data->output != NULL)
	{
		fprintf(prog_data->output, "Atout: ");
		game_print_family(prog_data->output, prog_data->family);
		fprintf(prog_data->output, "\n");
	}
	
	game_distribute_phase2(prog_data);
	
	set_points(prog_data);
	
	set_order_list(prog_data->players[2]);
	
	gtk_label_set_text(GTK_LABEL(prog_data->allwidgets[8]),
		"1/8");
	
	gtk_button_set_label(GTK_BUTTON(prog_data->allwidgets[2]), 
		"Next");
	
	/*gtk_spin_button_set_range(
		GTK_SPIN_BUTTON(prog_data->allwidgets[4]),
		1, 8);*/
	if(prog_data->player == 2)
	{
		/*gtk_widget_set_sensitive(prog_data->allwidgets[4], TRUE);*/
		set_moveable(prog_data->players[2], TRUE);
	}
	else
	{
		/*gtk_widget_set_sensitive(prog_data->allwidgets[4], FALSE);*/
		set_moveable(prog_data->players[2], FALSE);
	}
	
	prog_data->state = READY_DISTRIB2;
	game_play_ia(prog_data);
	
}

/* click_play : a mouse click on the 
 * game area means something should be 
 * done. The action here can be a card 
 * drop, a distributing process, or 
 * an end of a round.
 * In all case it should be related to 
 * the computer doing something. Not you.
 * But it's not the case at the moment.
 * --Need to be fixed--
 */
void click_play(struct _prog *prog_data)
{
	game_turn(prog_data, NULL);
	/*game_play_ia(prog_data);*/
}

/* action_click : the user clicked 
 * on something. And we need to know 
 * what he means. This is depending 
 * on the game state.
 * 
 * struct _prog *prog_data: see file def.h
 * 
 * What is used in the struct _prog:
 * 
 * prog_data->state : enum egstate, 
 * see file def.h for more details.
 * 
 * 
 */
void action_click(struct _prog *prog_data)
{
	if(prog_data->state == READY_PILE)
	{
		if(prog_data->output != NULL)
		{
			fprintf(prog_data->output,
				"Distribute phase 1 please!\n");
		}
		
		if((prog_data->points[0] > 100 || prog_data->points[1] > 100 )
		&& (prog_data->points[0] != prog_data->points[1]) )
		{

			free_cell_list(&prog_data->cells);
	

			prog_data->report = 0;
			
			prog_data->points[0] = 0;
			prog_data->points[1] = 0;
			
			prog_data->row = 2;

			gtk_table_resize(GTK_TABLE(prog_data->allwidgets[9]),
				prog_data->row, 2);
			
/*			gtk_table_attach_defaults(GTK_TABLE(prog_data->allwidgets[9]),
				gtk_label_new("0"), 
				0, 1, prog_data->row-1, prog_data->row);
			
			gtk_table_attach_defaults(GTK_TABLE(prog_data->allwidgets[9]),
				gtk_label_new("0"), 
				1, 2, prog_data->row-1, prog_data->row);
			
			gtk_widget_show_all(prog_data->allwidgets[0]);
	*/		
			
			
		}
		

		click_givecard(prog_data);
	}	
	/*else 
	if(prog_data->state == READY_DISTRIB1)
	{
		click_selectmaster(prog_data);
	}*/
	else
	if(prog_data->state == READY_DISTRIB2)
	{
		if(prog_data->output != NULL)
		{
			fprintf(prog_data->output,
				"Play !\n");
		}

		click_play(prog_data);
	}
	
	draw_container(prog_data);
}

/* event_start : user clicked on the Start 
 * labelled button. In the future this will 
 * be replaced by something else.
 * 
 * struct _prog *prog_data: see file def.h
 * 
 */
void event_start(GtkButton *button, gpointer data)
{
	struct _prog *prog_data = (struct _prog*)data;
	
		if(prog_data->output != NULL)
		{
			fprintf(prog_data->output,
				"Click on Button\n");
		}
	
	if(prog_data->state == READY_DISTRIB1)
	{
		if(prog_data->output != NULL)
		{
			fprintf(prog_data->output,
				"You don't want this card. Ok!\n");
		}
		game_choose_trump_next(prog_data);
		/*click_selectmaster(prog_data);*/
	}
	else
		action_click(prog_data);

}

/* motion_notify_event : the user drag the mouse. 
 * This means a mouse button is pressed and the 
 * cursor is moving.
 * 
 * struct _prog *prog_data: see fil def.h
 * 
 * What is needed in struct _prog:
 * 
 * prog_data->movecard : this is a pointer 
 * to a struct _movingcard defined in the 
 * def.h file. It will be NULL if there are 
 * no moving card at the moment. A moving 
 * card is a card which the user is dragging 
 * around on the game area. 
 * The struct contains informations about 
 * this card, like a pointer to the 
 * struct _card of this particular card. 
 * Also a separate struct _dim for the 
 * position of the dragged card exists 
 * because we need to recover the orginal 
 * position of the card if it can't be 
 * dropped. It's preferable to keep 
 * the original data in the struct _card, 
 * not in the newly allocated movecard _dim.
 * 
 */
gint motion_notify_event (GtkWidget *widget, GdkEventMotion *event, gpointer data)
{
	struct _prog *prog_data = (struct _prog*)data;
	
	if(prog_data->movecard)
	{
		prog_data->movecard->dim.x = 
			(gint)event->x -prog_data->movecard->dim.w;

		prog_data->movecard->dim.y = 
			(gint)event->y -prog_data->movecard->dim.h;
		
		draw_container(prog_data);
		
	}
	
	return TRUE;
}

/* find_moveable_card : when the user click 
 * on the game area, we need to find what 
 * card is candidate to be a moving card. 
 * This means scanning each GList of visible 
 * cards in the order of visibility. From the 
 * nearer to the user to the deeper.
 * A card which is on the area of the click 
 * event is not automaticaly a candidate if 
 * another card is hiding it.
 * 
 * GList *ptr: a GList containing cards element 
 * as struct _card (see file def.h for more 
 * details about this struct).
 * 
 * GdkEventButton *event: contains x,y 
 * position of the mouse click.
 * 
 * struct _prog *prog_data: see file def.h
 * 
 * What is needed in the struct _prog:
 * 
 * prog_data->movecard : a struct _movingcard
 * defined in def.h file, where we will 
 * record a pointer to the card selected in 
 * case the user keep the mouse button 
 * pressed. He wants to drag a card. We need 
 * to find it first. And keep a trace of it 
 * easily to make it move later. This is 
 * what the struct _movingcard is used for.
 * 
 * return gboolean: TRUE if we found a candidate.
 * FALSE otherwise.
 */
gboolean find_moveable_card(GList *ptr, GdkEventButton *event, 
	struct _prog *prog_data)
{
	struct _card *data_ptr;
	gboolean found = FALSE;
	
	while( ptr )
	{
		data_ptr = ptr->data;
		
		if(event->button == 1 && (gint) event->x >= data_ptr->dim.x 
			&& (gint) event->y >= data_ptr->dim.y 
			&& (gint) event->x < data_ptr->dim.x+data_ptr->dim.w
			&& (gint) event->y < data_ptr->dim.y+data_ptr->dim.h)
		{
			if(	data_ptr->moveable == TRUE && prog_data->movecard == NULL)
			{
				prog_data->movecard = (struct _movingcard*)
					malloc(sizeof( struct _movingcard) );
				
				if( prog_data->movecard )
				{
					prog_data->movecard->card = data_ptr;
					
					prog_data->movecard->dim.w = (gint)event->x - data_ptr->dim.x;
					prog_data->movecard->dim.h = (gint)event->y - data_ptr->dim.y;
					
					prog_data->movecard->dim.x = data_ptr->dim.x;
					prog_data->movecard->dim.y = data_ptr->dim.y;
					
					data_ptr->draw = FALSE;
					
					draw_container(prog_data);
				}
			}
			found = TRUE;
			ptr = NULL;
		}
		else
			ptr = ptr->next;
	}

	return(found);
}

/* button_press_event : user did press a mouse button.
 * 
 * prog_data->waiting: GList containing cards dropped 
 * on the table.
 * 
 * prog_data->players[2] : GList containing the user 
 * hand. The player corresponding to the human 
 * player is the third of the players array ([2]) 
 * and is represented at the bottom of the screen.
 * 
 */
void button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer data)
{
	struct _prog *prog_data = (struct _prog*)data;

	if(event->button == 1)
	{
		if( find_moveable_card(prog_data->waiting, event, prog_data) == FALSE)
			find_moveable_card(prog_data->players[2], event, prog_data);
	}
}

/* button_release_event : user released a pressed 
 * mouse button. This means we need to check if 
 * a card was selected. If true, then we want 
 * to check if it can be dropped here.
 * 
 * See file def.h for more details about 
 * structs and their fields.
 */
void button_release_event (GtkWidget *widget, GdkEventButton *event, gpointer data)
{
	struct _prog *prog_data = (struct _prog*)data;
	/*int xmin, ymin, xmax, ymax, w, h;*/
	int midx, midy;
	struct _card *drop = NULL;
	struct _target *zone = NULL;
	
	if(event->button == 1)
	{
		if(prog_data->movecard)
		{
			prog_data->movecard->card->draw = TRUE;
			
			if( ((prog_data->player == 2  && prog_data->player != prog_data->hashand )
			  || ( prog_data->player == prog_data->hashand && prog_data->menext == 2) )
			  && prog_data->players[2] != NULL)
			{
				midx = prog_data->movecard->dim.x+ 
					(prog_data->movecard->dim.w/2);
				midy = prog_data->movecard->dim.y+
					(prog_data->movecard->dim.h/2);
				
				zone = get_active_targetzone(prog_data->dropping);
				
				if(zone && midx > zone->dim.x && 
					midx < zone->dim.x + zone->dim.w &&
					midy > zone->dim.y && 
					midy < zone->dim.y + zone->dim.h)
				{
					/*gdk_drawable_get_size(prog_data->back, 
						&w, &h);
				
					xmin = prog_data->area->allocation.width/2;
					ymin = prog_data->area->allocation.height/2;
					
					xmin -= 50;
					ymin -= 60;
					xmax = xmin +100;
					ymax = ymin +120;
					
					if( prog_data->movecard->card->dim.y == (prog_data->area->allocation.height -h -10)
					&& prog_data->movecard->dim.x > xmin 
					&& prog_data->movecard->dim.x < xmax 
					&& prog_data->movecard->dim.y > ymin 
					&& prog_data->movecard->dim.y < ymax )*/
					{
						if(prog_data->state == READY_DISTRIB2 
						|| prog_data->state == READY_DISTRIB1)
						{
							drop = prog_data->movecard->card;
						}
					}
				}
			}
			free(prog_data->movecard);
			prog_data->movecard = NULL;


			if(drop != NULL)
			{
				if(prog_data->state == READY_DISTRIB1)
				{
					if(prog_data->output != NULL)
					{
						fprintf(prog_data->output,
							"You selected the card!\n");
					}
					if(prog_data->player == 2)
						click_selectmaster(prog_data, 2, 
						((struct _card*)prog_data->waiting->data)->family);
			
				}
				else
					game_turn(prog_data, drop);
				/*game_play_ia(prog_data);*/
			}
			draw_container(prog_data);
			

		}
		else
		{
			if(prog_data->output != NULL)
			{
				fprintf(prog_data->output,
					"Click on game area\n");
			}
			
			action_click(prog_data);
		}
	}
}

/* realisation : the game area has been 
 * initilialised by Gtk, we now needs to 
 * draw it. This means allocating a 
 * GtkPixmap to draw to, then loading 
 * cards (because we need to know the window 
 * depth before loading any images).
 * 
 * Then we can set the position of the 
 * dropping zone area, because we need 
 * to know the size of the area before 
 * positioning it. At least it's true 
 * for the middle area of the table.
 * At the moment it's the only dropping 
 * area created.
 * 
 * Then we can start the game, that means 
 * preparing everything for the game to 
 * be played. As an example there is the 
 * prog_data->pile GList that needs to be 
 * mixed. 
 * 
 * See game_start() function for more details 
 * about what needs to be done. (file game.c)
 * 
 * return gboolean: see Gtk documentation 
 * about realize event of a gdk-drawing area.
 */
gboolean realisation( GtkWidget *aire_de_dessin, gpointer data )
{
	struct _prog *prog_data = (struct _prog*) data;
	
	alloc_prog(prog_data);
	
	load_cards("pixmaps",
			prog_data,
			aire_de_dessin);
	
	position_targetzone(prog_data);
	
	game_start(prog_data);
	
	return TRUE;
}

/* rafraichissement : refresh the game 
 * area.
 * 
 * return gboolean: see Gtk documentation 
 * about expose-event
 */
gboolean rafraichissement( GtkWidget *aire_de_dessin, GdkEventExpose *event, gpointer data)
{
	draw_container((struct _prog*)data );
	
	return FALSE;
}

/* 
 * return gboolean: see Gtk documentation 
 * about destroy event
 */
gboolean quit(GtkWidget *window, gpointer data)
{
	gtk_main_quit();
	
	return TRUE;
}

/* resize: the window size has changed. 
 * We need to draw and position everything 
 * again.
 * 
 * see Gtk documentation about check-resize 
 * event for more informations.
 */
void resize(GtkContainer *container, gpointer data)
{
	struct _prog *prog_data = (struct _prog*) data;
	
	gboolean animation;
	
	if( prog_data->area != NULL && 
		prog_data->area->window != NULL )
	{
		alloc_pixmap(prog_data);
		
		animation = prog_data->animation;
		if(animation == TRUE)
			prog_data->animation = FALSE;
		
		game_middle_position(prog_data);
		
		game_hand_position(prog_data, -1);
		if(animation == TRUE)
			prog_data->animation = TRUE;
		
		position_targetzone(prog_data);
		
		draw_container(prog_data);
	
	}
}

void dialog_response_event(GtkDialog *dialog, gint rid, gpointer data)
{
	if(rid == GTK_RESPONSE_CLOSE)
		gtk_widget_destroy(GTK_WIDGET(dialog));
}
