/*
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 2 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */
#include <string.h>
#include <math.h>
#include <gtk/gtk.h>
#include "select.h"
#include "throw.h"
#include "position.h"
#include "set.h"
#include "game.h"
#include "free.h"
#include "draw.h"
#include "callback.h"
#include "def.h"

/* game_play_ia: play automagically. That's 
 * what we call Artificial Intelligence.
 * Indeed that's not intelligent at all.
 */ 
void game_play_ia(struct _prog *prog_data)
{
	while(prog_data->player != 2 
	&& prog_data->players[prog_data->player] != NULL)
	{
		game_turn(prog_data, NULL);
	}
}

/* game_middle_position: set the position (x,y)
 * of each card in the list represented at the 
 * center of the game area (the table).
 */ 
void game_middle_position(struct _prog *prog_data)
{
	GList *ptr;
	
	struct _card *data;
	
	int player = prog_data->hashand;
	
	if(player != -1)
	{
		for(ptr = g_list_last(prog_data->waiting); ptr; ptr = ptr->prev)
		{
			data = ptr->data;
			
			game_position_middle(data, prog_data, player);
			
			select_player_next(&player);

		}
	}
}

/* game_hand_position: set the position (x,y)
 * of the cards in each player hand.
 */ 
void game_hand_position(struct _prog *prog_data, int player)
{
	int x, y, w, h;
	enum eposition ep;
	
	int coef, start, max;
	
	gdk_drawable_get_size(prog_data->back, 
			&w, &h);
	
	if(player == -1)
	{
		start = 0;
		max = 4;
	}
	else
	{
		start = player;
		max = player+1;
	}
	
	for(player = start; player < max; player++)
	{
	
		switch(player)
		{
			case 0:
			y = 10;
			coef = 1;
			x = (prog_data->area->allocation.width - w) / 3;
			ep = EP_HORIZONTAL;
			break;
			
			case 1:
			x = prog_data->area->allocation.width - w -10;
			coef = 1;
			y = (prog_data->area->allocation.height - h) / 2;
			ep = EP_VERTICAL;
			break;
			
			case 2:
			y = prog_data->area->allocation.height - h -10;
			coef = -1;
			x = (prog_data->area->allocation.width - w)*2 / 3;
			ep = EP_HORIZONTAL;
			break;
			
			case 3:
			x = 10;
			coef = -1;
			y = (prog_data->area->allocation.height - h)*2 / 3;
			ep = EP_VERTICAL;
			break;
		}
		
		position_list(prog_data->players[player], 
			x, y, ep, coef, prog_data);
		
	}
}

/* game_send: distributing cards function.
 * This will send cards from one list to 
 * players list. Each player will take 
 * [the value of mod] cards.
 * 
 * int mod: how many cards per player.
 * 
 * int max: how many cards we want to send 
 * (total)
 * 
 * GList *ptr: the list where we take the 
 * cards.
 */ 
GList* game_send(struct _prog *prog_data, int mod, int max,
	GList *ptr)
{
	int player = prog_data->distributer;
	int i, modifier;
	GList *last;
	
	select_player_next(&player);
	
	for(modifier = 0, i = 1; i <= max; i++)
	{
		if(ptr != NULL)
		{
			throw_to(&prog_data->players[player],
				(struct _card*) ptr->data);
			
			if(player == 2)
			{
				((struct _card*) ptr->data)->draw_face = TRUE;
			}
			
			/* at the second distribution phase, 
			 * we need to send three cards to each player 
			 * except the one that choose the trump family.
			 * This one will take only two cards.
			 * That's why we do i++ here, so we won't be 
			 * distributing [the value of mod] cards to him, 
			 * but instead [[the value of mod]-1] cards. */
			if(player == prog_data->master && modifier == 0)
			{
				i++;
				modifier = 1;
			}
				
			if(i % mod == 0)
			{
				select_player_next(&player);
			}	
			
			
			last = ptr;
			ptr = ptr->next;

			prog_data->pile = g_list_remove(prog_data->pile, 
				last->data);
			
		}
		else i = max+1;
	}
	
	return(ptr);
}

/* game_distribute_phase1 : send five cards to 
 * each player. (Three first, and two)
 */ 
void game_distribute_phase1(struct _prog *prog_data)
{
	if(prog_data->players && prog_data->pile)
	{
		prog_data->pile = game_send(prog_data, 3, 12,
			prog_data->pile);
		
		prog_data->pile = game_send(prog_data, 2, 8,
			prog_data->pile);
		
		game_hand_position(prog_data, -1);
	}
}

/* game_distribute_phase2 : send remaining card in 
 * the pile to the players. The card which was 
 * proposed is to be sent to the master player.
 */ 
void game_distribute_phase2(struct _prog *prog_data)
{
	if(prog_data->players && prog_data->pile)
	{
		prog_data->pile = game_send(prog_data, 3, 12,
			prog_data->pile);
		
		if(prog_data->waiting)
		{
			struct _card *data_ptr = (struct _card*) prog_data->waiting->data;
			
			if(prog_data->master != 2)
				data_ptr->draw_face = FALSE;
			
			data_ptr->moveable = FALSE;
		
			throw_to(&prog_data->players[prog_data->master],
				data_ptr);
		
			prog_data->waiting = g_list_remove(prog_data->waiting, 
				data_ptr);
		}
		
		game_hand_position(prog_data, -1);
		
		set_moveable(prog_data->players[2], TRUE);
	}
}

/* game_finish_phase1 : just take one card from 
 * the pile and put it on the top of the pile, 
 * with the value displayed to the user. 
 * This is the proposed trump card.
 */ 
void game_finish_phase1(struct _prog *prog_data)
{
	GList *pile = prog_data->pile;
	
	if(pile != NULL)
	{
		struct _card *data_ptr = (struct _card*) pile->data;
		
		throw_to(&prog_data->waiting,
			data_ptr);
		
		data_ptr->draw_face = TRUE;
		data_ptr->moveable = TRUE;
		
		if(prog_data->output)
		{
			fprintf(prog_data->output,
				"Proposed card is ");
			game_print_card(prog_data->output, data_ptr);
			fprintf(prog_data->output, "\n");
			
		}
		
		prog_data->pile = g_list_remove(prog_data->pile, 
				data_ptr);
	}
}

/* game_drop_middle: there should be four 
 * cards on the table when calling this 
 * function. It means we need to know 
 * what team wins the turn, and send 
 * all of those cards to the winner team 
 * list.

 * Note: indeed the winner player of
 * this turn has already been determined, 
 * it's the 'menext' value in struct _prog
 * That's why we don't need to find the 
 * winner team here, we know it already
 * (it's [menext MODULO 2])
 */ 
void game_drop_middle(struct _prog *prog_data)
{
	struct _card *data_ptr;
	GList *ptr;
	int winner = -1;
	
	if(prog_data->menext >= 0)
	{
		winner = prog_data->menext % 2;
		
		if(prog_data->output != NULL)
		{
			fprintf(prog_data->output, "L'équipe %d", winner);
			if(winner == 0)
				fprintf(prog_data->output, "(0 = haut, 2 = vous)");
			else
				fprintf(prog_data->output, "(1 = droite, 3 = gauche)");
			fprintf(prog_data->output, " empoche le pli\n");
		}
		
		ptr = prog_data->waiting;
		
		while( ptr )
		{
			data_ptr = (struct _card *)ptr->data;
			
			data_ptr->draw = FALSE;
			
			throw_to(&prog_data->teams[winner], data_ptr);
			
			ptr = ptr->next;
			
		}
		
		g_list_free(prog_data->waiting);
		
		prog_data->waiting = NULL;
	}
	else if(prog_data->output != NULL)
	{
		fprintf(prog_data->output, 
			"Erreur: impossible de déterminer l'équipe qui remporte le pli\n");
	}
}

/* game_position_middle : set the position (x,y) 
 * of a card on the table. Each card is 
 * positionned in relation to the player 
 * who dropped it.
 */ 
void game_position_middle(struct _card *card, 
	struct _prog *prog_data,
	int player)
{
	int midx, midy;
	
	midx = (prog_data->area->allocation.width - card->dim.w)/2;
	midy = (prog_data->area->allocation.height - card->dim.h)/2;
	
	switch(player)
	{
		case 0:
		midy -= 25;
		break;
		
		case 1:
		midx += 25;
		break;
		
		case 2:
		midy += 25;
		break;
		
		case 3:
		midx -= 25;
		break;
	}
	
	if(card->dim.x != midx || card->dim.y != midy)
	{
		if(prog_data->animation)
			position_move_card(card, midx, midy, prog_data);
		

		card->dim.x = midx;
		card->dim.y = midy;
		draw_container(prog_data);
	}
}

/* game_print_family: that's in french atm,
 * feel free to translate.
 */ 
void game_print_family(FILE *stream, int family)
{
	switch(family)
	{
		case 0:
		/* diamond */
		fprintf(stream, "carreau");
		break;
		case 1:
		/* heart */
		fprintf(stream, "coeur");
		break;
		case 2:
		/* club */
		fprintf(stream, "pique");
		break;
		case 3:
		/* spade */
		fprintf(stream, "trefle");
		break;
	}

}

/* game_print_card: print the card as 
 * a nominative value.
 * Example: ace of diamond
 * Feel free to translate.
 */ 
void game_print_card(FILE *stream, struct _card *card)
{
	switch(card->num)
	{
		case 0:
		/* ace */
		fprintf(stream, "as");
		break;
		
		case 6:
		case 7:
		case 8:
		case 9:
		fprintf(stream, "%d", card->num+1);
		break;
		
		case 10:
		/* jack */
		fprintf(stream, "valet");
		break;
		
		case 11:
		/* queen */
		fprintf(stream, "dame");
		break;
		
		case 12:
		/* king */
		fprintf(stream, "roi");
		break;
	}
	
	/* of */
	fprintf(stream, " de ");
	
	game_print_family(stream, card->family);
}

/* game_get_family_best: what is the most 
 * powerfull card we get on a GList in 
 * a particular family only.
 */ 
int game_get_family_best(int family, GList *lst)
{
	int nb = -1, i, best = 0;
	
	struct _card *data;
	
	for(i=0; lst; i++)
	{
		
		data = (struct _card*)lst->data;
		
		if(data->family == family)
		{
			if(data->points > best)
			{
				best = data->points;
				nb = i;
			}
		}
		/*else
		if(data->family == majorfamily && best == 0)
		{
			best = data->points;
			family = data->family;
			nb = i;
		}*/

		lst = lst->next;
	}
	
	return(nb);
}

gboolean game_find_card(int family, int num, GList *lst)
{
	gboolean found = FALSE;
	struct _card *data;
	
	while(lst)
	{
		data = lst->data;
		
		if(data->family == family 
			&& data->num == num)
		{
			found = TRUE;
			lst = NULL;
		}
		else
			lst = lst->next;
	}
	
	return(found);
}

/* game_get_best: what is the most 
 * powerful card on a list, related 
 * to a family, but not only. 
 * If we found a trump card, it will 
 * probably be better than something 
 * else. That's what majorfamily is 
 * all about.
 * 
 * Note: if the family value is -1, 
 * then the first card family is 
 * taken as a reference to find more 
 * powerfull cards. Don't forget 
 * that if majorfamily is set (other 
 * than -1), and if it's different 
 * than the value of family, any 
 * card of the majorfamily is better 
 * then the first card.
 * 
 * That function is heavily used to 
 * find if a card dropped on the 
 * table is better than others.
 * 
 * We just find what is the best 
 * card on the table. Then we check 
 * this card against the one to be 
 * dropped.
 */ 
struct _card* game_get_best(int *best, int *family, 
	int majorfamily, GList *lst)
{
	struct _card *data, *first = NULL;
	
	lst = g_list_last(lst);
	
	while(lst)
	{
		
		data = (struct _card*)lst->data;
		
		if(*family == -1)
		{
			first = data;
			*best = data->points;
			*family = data->family;
		}
		else
		if(data->family == *family)
		{
			if(data->points > *best)
				*best = data->points;
		}
		else
		if(data->family == majorfamily)
		{
			*best = data->points;
			*family = data->family;
		}
			
		lst = lst->prev;
	}
	
	return(first);
}

/* game_check_cardisbetter: check if a card 
 * is more powerfull than the most powerfull
 * card on a GList
 */ 
gboolean game_check_cardisbetter(struct _card *card, GList *lst,
	int majorfamily)
{
	int best = 0, family = -1;
	gboolean better = FALSE;
	
	struct _card *first;
	
	first = game_get_best(&best, &family, majorfamily, lst);
	
	if(best > 0)
	{
		if(card->family == family)
		{
			if(card->points > best)
				better = TRUE;
		}
		else if(card->family == majorfamily)
		{
			better = TRUE;
		}
	}
	
	return(better);
}

/* game_ia_select_card: what the computer should 
 * play. This is where every decision should be 
 * made. 
 * 
 * 
 * Note: the rules of the game should be excluded 
 * from here and put on another function, because 
 * the human player is also affected by the rules.
 * --Need to be Fixed--
 */ 
int game_ia_select_card(struct _prog *prog_data)
{
	int nb=0, best = 0, family = -1;
	
	struct _card *first = NULL;
	
	if(prog_data->waiting != NULL)
	{
		first = game_get_best(&best, &family, prog_data->family, 
			prog_data->waiting);
	}
	else
	{
		/* computer will drop his first card, no 
		 * matter what it is. Because no card has 
		 * been dropped on the table on this turn 
		 * yet, he can play anything. He should 
		 * play cleverly in the future.
		 * --Need to be Fixed-- */
		
	}
	
	if(first != NULL)
	{
		/* Here he plays his best card in a family, 
		 * if he has one in it. */
		nb = game_get_family_best(first->family, 
			prog_data->players[prog_data->player]);
		
		if(nb < 0)
		{
			/*fprintf(prog_data->output, 
				"Le joueur %d n'a pas de ", prog_data->player);
			game_print_family(prog_data->output, first->family);
			*/
			if(prog_data->family != first->family)
			{
				nb = game_get_family_best(prog_data->family, 
				prog_data->players[prog_data->player]);
				/*if(nb > 0)
					fprintf(prog_data->output, " mais il a de l'atout");
				else
					fprintf(prog_data->output, " et n'a pas d'atout");*/
			}
			/*fprintf(prog_data->output, "\n");*/
			
		}
		
		if(nb < 0)
			nb = 0;
	}
	
	return(nb);
}

/* game_start: makes the main list  
 * of cards, called the pile.
 * That's where we take care of 
 * preparing that pile and the 
 * status of each card before 
 * starting a new round. 
 */ 
void game_start(struct _prog *prog_data)
{
	throw_card_list( prog_data->all );
	
	throw_cards(prog_data);
	
	prog_data->state = READY_PILE;
	
	prog_data->turn = 8;
	
	gtk_button_set_label(GTK_BUTTON(prog_data->allwidgets[2]), 
		"Start");
		
	gtk_image_set_from_pixbuf(GTK_IMAGE(prog_data->allwidgets[5]),
		prog_data->icons[4]);
	
	gtk_label_set_text(GTK_LABEL(prog_data->allwidgets[7]), "N/a");
	gtk_label_set_text( GTK_LABEL(prog_data->allwidgets[8]), "None");
	
	gtk_label_set_text( GTK_LABEL(prog_data->allwidgets[10]), NULL);
	gtk_label_set_text( GTK_LABEL(prog_data->allwidgets[11]), NULL);
		
	gtk_widget_set_sensitive(prog_data->allwidgets[2], TRUE);
}

/* game_totalcards: calculate the 
 * total value of points in a GList. 
 */ 
int game_totalcards(GList *lst)
{
	int total = 0;
	
	struct _card *data;
	
	while(lst)
	{
		data = lst->data;
		
		 /* Warning: the points value of cards 
		 * are not set to the real value 
		 * of the card in the game, but at 
		 * a more powerfull value.
		 * The real value is [value/10] */
		total += data->points/10;
		
		lst = lst->next;
	}
	
	return(total);
}

/* game_restart: reset state and int values 
 * of the game before each new rounds.
 */ 
void game_restart(struct _prog *prog_data)
{
	prog_data->state = LOADING;
	prog_data->player = -1;
	prog_data->master = -1;
	prog_data->family = -1;
	
	prog_data->hashand = -1;
	prog_data->menext = -1;
	
}

/* game_turn: so complicated. 
 * --Need to be simplified--
 */ 
void game_turn(struct _prog *prog_data, struct _card *card)
{
	int cardnb;
	
	if(prog_data->player == prog_data->hashand)
	{
		if(prog_data->waiting != NULL)
		{
			char str[4];
			
			sprintf(str, "%d/8", (9-prog_data->turn) );
				
			gtk_label_set_text(
				GTK_LABEL(prog_data->allwidgets[8]), str);

			game_drop_middle(prog_data);
			
	
			if(prog_data->menext >= 0)
			{
				prog_data->hashand = prog_data->menext;
				prog_data->player = prog_data->menext;
			}
		}
	}
	
	if(prog_data->turn > 0)
	{
		if(card == NULL && prog_data->player != 2)
		{
			/*if(prog_data->player == 2)
				cardnb = gtk_spin_button_get_value_as_int(
					GTK_SPIN_BUTTON(prog_data->allwidgets[4]))-1;
			else*/
				cardnb = game_ia_select_card(prog_data);
			
			card = throw_card_x(prog_data->players[prog_data->player], 
				cardnb);
		}
		
		

		if(card != NULL)
		{
			if(prog_data->output != NULL)
			{
				fprintf(prog_data->output, 
					"Joueur %d joue la carte ", 
					prog_data->player);
				game_print_card(prog_data->output, card);
				fprintf(prog_data->output, "\n");
			}
			
			if(prog_data->waiting != NULL 
			&& game_check_cardisbetter(card, prog_data->waiting, 
				prog_data->family))
			{
				if(prog_data->output != NULL)
					fprintf(prog_data->output, 
						"Joueur %d prend la main\n", 
						prog_data->player);
				
				prog_data->menext = prog_data->player;
			}
			
			throw_to(&prog_data->waiting, card);
			
			card->draw_face = TRUE;
			card->moveable = FALSE;
			
			if(card->points == 40 
				&& card->family == prog_data->family)
			{
				/* Look for the trump queen, 
				 * because this player just dropped 
				 * the trump queen. 
				 * If he got both, then it's a "Belote" 
				 * and 20 points goes to the team. */
				if(game_find_card(card->family, 11, 
					prog_data->players[prog_data->player]) == TRUE)
				gtk_label_set_text( GTK_LABEL(
					prog_data->allwidgets[10+(prog_data->player%2)]),
					"20");
			}
			
			prog_data->players[prog_data->player] = g_list_remove(
				prog_data->players[prog_data->player], 
				(gconstpointer) card);
			
			game_hand_position(prog_data, prog_data->player);
		
		
			
			game_position_middle(card, prog_data, prog_data->player);
			
			select_player_next(&prog_data->player);
			
			if(prog_data->player == 2)
			{
				set_moveable(prog_data->players[2], TRUE);
				/*gtk_widget_set_sensitive(prog_data->allwidgets[4], TRUE);*/
			}
			else
			{
				set_moveable(prog_data->players[2], FALSE);
				/*gtk_widget_set_sensitive(prog_data->allwidgets[4], FALSE);*/
			}
			
			if(prog_data->player == prog_data->hashand)
			{
				struct _card *card = NULL;
				int player;
				int i;
				
				if(prog_data->menext == 2)
				{
					/*gtk_widget_set_sensitive(prog_data->allwidgets[4], TRUE);*/
					set_moveable(prog_data->players[2], TRUE);
				}
				else
				{
					/*gtk_widget_set_sensitive(prog_data->allwidgets[4], FALSE);*/
					set_moveable(prog_data->players[2], FALSE);
				}
				
				if(prog_data->waiting)
				{
					/*((struct _card*)g_list_last(prog_data->waiting)->data)->family;*/
					/*if(prog_data->output)
						fprintf(prog_data->output, 
							"family %d, player has hand %d\n", family, prog_data->hashand);
					*/
					
					/* find the first card, to get the family of this turn */
					/*family = -1;
					card = game_get_best(&points, 
						&family, 
						prog_data->family, 
						prog_data->waiting);
					
					if(card)*/
					{
						/* now find the number of the card (from 0 to 3) 
						 * which is the best card in the list.
						 * This is pretty hard to know :
						 * Get the player who dropped the first 
						 * card and increment i till player == menext 
						 * Because menext is the winner player 
						 * of this turn. */
						
						player = prog_data->hashand;
						select_player_prev(&player);
						if(prog_data->output)
						fprintf(prog_data->output, 
							"hashand %d, menext %d\n", prog_data->hashand, prog_data->menext);
						for(i = 0; player != prog_data->menext ; i++)
						{
							select_player_prev(&player);
						}
						
						/*family = card->family; */
						card = throw_card_x(prog_data->waiting, i);
						
						if(card)
						{
							card->blink = TRUE;
						}
					}
						
				}
				
				prog_data->turn--;
				if(prog_data->turn <= 0)
				{
					gtk_button_set_label(GTK_BUTTON(prog_data->allwidgets[2]), "Finish");
				}
			}
		}
	}
	else
	{
		int Pts[2];
		char str[5];
		GtkWidget *cell;
		int masterteam = prog_data->master%2;
		
		prog_data->state = ENDGAME;
		/*gtk_widget_set_sensitive(prog_data->allwidgets[2], FALSE);
		gtk_widget_set_sensitive(prog_data->allwidgets[4], FALSE);*/
		set_moveable(prog_data->players[2], FALSE);
		
		Pts[0] = game_totalcards(prog_data->teams[0]);
		Pts[1] = game_totalcards(prog_data->teams[1]);
		
		/* Dix de der */
		Pts[prog_data->menext%2] += 10;
		
		if(prog_data->output != NULL)
		{
			fprintf(prog_data->output, 
				"Points des cartes: [0,2] %d (vous) contre (eux) %d [1,3]\n",
				Pts[0],
				Pts[1]);
			
			fprintf(prog_data->output,
				"master was %d, menext is %d\n",
				(prog_data->master),
				prog_data->menext );
		}
		
		if(Pts[masterteam] >= Pts[!masterteam])
		{
			if(Pts[!masterteam] == 0)
				Pts[masterteam] = 25;
			else
			{
				Pts[0] = (int)round(Pts[0]/(float)10);
				Pts[1] = (int)round(Pts[1]/(float)10);
			}
			
			if( game_totalcards(prog_data->teams[0]) == game_totalcards(prog_data->teams[1]))
			{
				prog_data->report += Pts[masterteam];
				
				if(prog_data->output)
				{
					fprintf(prog_data->output,
						"Egalité! Report des points de l'equipe de master(%d)\n%d points, (%d total)\n",
						prog_data->master, 
						Pts[masterteam],
						prog_data->report);
				}
				
				Pts[masterteam] = 0;
			}
			else
			{
				Pts[masterteam] += prog_data->report;
			
				prog_data->report = 0;
			}
		}
		else
		{
			Pts[masterteam] = 0;
			Pts[!masterteam] = 16 +prog_data->report;
			prog_data->report = 0;
		}
		
		prog_data->points[0] += Pts[0];
		prog_data->points[1] += Pts[1];
		
		prog_data->row++;
		
		gtk_table_resize(GTK_TABLE(prog_data->allwidgets[9]), 
			prog_data->row, 2);
			
		sprintf(str, "%d", prog_data->points[0]);
		cell = gtk_label_new(str);
		
		prog_data->cells = g_list_prepend(prog_data->cells, 
			(gpointer) cell );
			
		gtk_table_attach_defaults(GTK_TABLE(prog_data->allwidgets[9]),
			cell, 
			0, 1, prog_data->row-1, prog_data->row);
		
		sprintf(str, "%d", prog_data->points[1]);
		cell = gtk_label_new(str);
		
		prog_data->cells = g_list_prepend(prog_data->cells, 
			(gpointer) cell );
		
		gtk_table_attach_defaults(GTK_TABLE(prog_data->allwidgets[9]),
			cell, 
			1, 2, prog_data->row-1, prog_data->row);
		
		gtk_widget_show_all(prog_data->allwidgets[0]);
		
		if(Pts[0] > Pts[1])
			gtk_label_set_text(GTK_LABEL(prog_data->allwidgets[6]), "You Win");
		else
			gtk_label_set_text(GTK_LABEL(prog_data->allwidgets[6]), "Computer Wins");
		
		/*gtk_scrolled_window_set_vadjustment(
		GTK_SCROLLED_WINDOW(prog_data->allwidgets[1]),
		GTK_ADJUSTMENT(gtk_adjustment_new(1, 0, 1, 0.1, 0.5, 0.1)));*/
		
		
		if((prog_data->points[0] > 100 || prog_data->points[1] > 100) 
		&& prog_data->points[0] != prog_data->points[1])
		{
			GtkWidget *dialog;
			char winstr[50];
			
			if(prog_data->points[0] > prog_data->points[1])
				strcpy(winstr, "Congratulation, you won this game!");
			else
				strcpy(winstr, "Sorry, you have lost this game!");
			
			if(prog_data->output)
				fprintf(prog_data->output,
					"\nFin de Partie !!\n\n\n");
			
			dialog = gtk_message_dialog_new(
				GTK_WINDOW(prog_data->allwidgets[0]), 
				GTK_DIALOG_MODAL,
				GTK_MESSAGE_INFO,
				GTK_BUTTONS_CLOSE,
				winstr, NULL);
			
			g_signal_connect(G_OBJECT(dialog), "response", G_CALLBACK(dialog_response_event), prog_data);
			
			gtk_widget_show_all(dialog);
			
		}
			
		
		
		if(prog_data->allwidgets && prog_data->allwidgets[5])
		{
			if(gtk_image_get_pixbuf(GTK_IMAGE(prog_data->allwidgets[5])))
				gtk_image_clear(GTK_IMAGE(prog_data->allwidgets[5]) );
		}
		gtk_label_set_text( GTK_LABEL(prog_data->allwidgets[8]), "None");
		free_clear(prog_data);
		game_restart(prog_data);
		game_start(prog_data);
		
	}
}

void game_count_cards(GList *lst, int family, 
	int *pointstrump, int *pointsnottrump,
	int *counttrump, int *countnottrump,
	int *best)
{
	struct _card *card;
	int points;
	
	while(lst)
	{
		card = (struct _card *)lst->data;
		
		if( card->family == family)
		{
			points = set_master_points(card->num);
			if(points > *best)
				*best = points;
			*pointstrump += points;
			(*counttrump)++;
		}
		else
		{
			*pointsnottrump += set_notmaster_points(card->num);
			(*countnottrump)++;
		}
		
		lst = lst->next;
	}
	
}

/* 
 */
gboolean game_ia_select_trump(struct _prog *prog_data, int player)
{
	gboolean select = FALSE;
	struct _card *card = ((struct _card*)prog_data->waiting->data);
	int pointstrump = 0, pointsnottrump = 0, 
	counttrump = 0, countnottrump = 0, best = 0;
	
	game_count_cards(prog_data->players[player], 
		card->family, &pointstrump, &pointsnottrump,
		&counttrump, &countnottrump, &best);
	
	if(counttrump > 1)
	{
		pointstrump += set_master_points(card->num);
		
		/* from 140 to 225 
		 * lesser means more crazyness */
		if(pointstrump >= 225)
		{
			if(prog_data->output)
				fprintf(prog_data->output, "Player %d is susceptible to take the card [%d],[%d]\n", prog_data->player, pointsnottrump, pointstrump);
			
			if( pointsnottrump+pointstrump > 270 
			&& (pointsnottrump/countnottrump) + (pointstrump/counttrump) >= 60
			&& best >= 110)
			{
				select = TRUE;
			}
		}
	}
	
	return(select);
}

/*
 */
void game_choose_trump_next(struct _prog *prog_data)
{
	gboolean play = FALSE;
	int player, trump, i;
	
	select_player_next(&prog_data->player);
	
	for(i =0; i< 4; i++)
	{
		if(prog_data->player != prog_data->menext)
		{
			if(prog_data->output != NULL)
			{
				fprintf(prog_data->output,
					"Computer turn to choose (%d)\n", prog_data->player);
			}
			play = game_ia_select_trump(prog_data, prog_data->player);
			if(play == TRUE)
			{
				player = prog_data->player;
				prog_data->player = prog_data->menext;
				trump = ((struct _card *)prog_data->waiting->data)->family;
				i = 5;
			}
			
			select_player_next(&prog_data->player);
		}
		else
			i = 5;
	} 
	
	if(play == TRUE)
	{
		click_selectmaster(prog_data, player, trump);
	}
	else
	{
		if(prog_data->output != NULL)
		{
			fprintf(prog_data->output,
				"No one seems to want this card, restart!\n");
		}

		free_clear(prog_data);
		game_restart(prog_data);
		game_start(prog_data);
		
	}
}

/* game_choose_trump : first phase of the game, 
 * we need to select the trump.
 */
void game_choose_trump(struct _prog *prog_data)
{
	int i, trump;
	
	trump = ((struct _card*)prog_data->waiting->data)->family;
	
	if(prog_data->output != NULL)
	{
		fprintf(prog_data->output,
			"Please choose trump\n");
	}

	for(i = 0; i < 4; i++)
	{
		if(prog_data->player == 2)
		{
			i = 5;
			if(prog_data->output != NULL)
			{
				fprintf(prog_data->output,
					"It's your turn to choose!\n");
			}

			gtk_widget_set_sensitive(prog_data->allwidgets[2], TRUE);
			
		}
		else
		{
			if(prog_data->output != NULL)
			{
				fprintf(prog_data->output,
					"Computer turn to choose (%d)\n", prog_data->player);
			}
			if(game_ia_select_trump(prog_data, prog_data->player) == TRUE)
			{
				
				click_selectmaster(prog_data, prog_data->player, trump);
				i=5;
			}
			else
				select_player_next(&prog_data->player);
		}
	}
}
