/*
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 2 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

#include <gtk/gtk.h>
#include "throw.h"
#include "def.h"

/* set_draw: set the draw flag on 
 * each card in a list.
 */
void set_draw(GList *ptr, gboolean value)
{
	struct _card *data_ptr;
	
	for(; ptr; ptr = ptr->next)
	{
		data_ptr = ptr->data;
		
		data_ptr->draw = value;
	}
}

/* set_moveable: set the moveable flag 
 * on each card in a list.
 */
void set_moveable(GList *ptr, gboolean value)
{
	struct _card *data_ptr;
	
	for(; ptr; ptr = ptr->next)
	{
		data_ptr = ptr->data;
		
		data_ptr->moveable = value;
	}
}

/* set_status: change state of 
 * each card in a list.
 */
void set_status(GList *ptr, enum estatus s)
{
	struct _card *data_ptr;
	
	for(; ptr; ptr = ptr->next)
	{
		data_ptr = ptr->data;
		
		data_ptr->status = s;
	}
}

/* set_notmaster_points: take 
 * care of setting points for 
 * a card which is not in the 
 * trump family.
 */
int set_notmaster_points(int num)
{
	int value = 0;
	
	switch(num)
	{
		case 0:
		value = 110;
		break;
		
		case 6:
		case 7:
		case 8:
		value = num+1;
		break;
		
		case 9:
		value = 100;
		break;
		
		case 10:
		value = 20;
		break;
		
		case 11:
		value = 30;
		break;
		
		
		case 12:
		value = 40;
		break;
	}
	
	return(value);
}

/* set_master_points: for each card, 
 * the value of the points changes. 
 * That will take care to set the 
 * correct value for a card in the 
 * trump color.
 */
int set_master_points(int num)
{
	int value = 0;
	
	switch(num)
	{
		case 0:
		value = 110;
		break;
		
		case 6:
		case 7:
		value = num+1;
		break;

		case 8:
		value = 140;
		break;
		
		case 9:
		value = 100;
		break;
		
		case 10:
		value = 200;
		break;
		
		case 11:
		value = 30;
		break;
		
		case 12:
		value = 40;
		break;
	}
	
	return(value);
}

/* set_individual_list: set points to the 
 * cards in the list. 
 */
void set_individual_list(GList *lst, int family)
{
	struct _card *data;
	
	while(lst)
	{
		data = (struct _card*)lst->data;
		
		if(data->family == family)
		{
			data->points = set_master_points(data->num);
		}
		else
		{
			data->points = set_notmaster_points(data->num);
		}
		
		lst = lst->next;
	}
}

/* set_points: set the value of every 
 * available card in this game. This 
 * will just go through every player 
 * list.
 */
void set_points(struct _prog *prog_data)
{
	int i;
	
	for(i = 0; i < 4; i++)
	{
		set_individual_list(prog_data->players[i], 
			prog_data->family);
		
	}
}

void swap(struct _card *one, struct _card *two, GList *lst)
{
	struct _card *tmp;
	
	GList *prev;
	
	while(lst)
	{
		tmp = (struct _card *)lst->data;
		
		prev = lst;
		lst = lst->next;
		
		if(tmp == one && lst)
		{
			prev->data = (gpointer) two;
			lst->data =  (gpointer) one;
			
			lst = NULL;
		}
		
	}
}

void set_order_list(GList *lst)
{
	struct _card *data, *next;
	GList *tmp;
	int values[8], i = 0, tval;
	
	for(tmp = lst; tmp; tmp = tmp->next)
	{
		data = (struct _card *)tmp->data;
		
		values[i] = (data->family * 1000) + data->points;
		i++;
		
	}

	for(i=0; i < 7; i++)
	{
		if(i < 0)
			i = 0;
		if(values[i] > values[i+1])
		{
			data = throw_card_x(lst, i);
			next = throw_card_x(lst, i+1);
			if(data && next)
			{
				tval = values[i];
				values[i] = values[i+1];
				values[i+1] = tval;
				swap(data, next, lst);
				i = i-2;
			}
		}
	}
}
